/// @file       webserver3.c
/// @copyright  Copyright (c) 2014 Toradex AG
/// @author     $guc$
/// @date       $2014-2-24$
/// @brief      This application shows how to implement a Web-server on Toradex module. In this
///             demo AJAX, HTML5 Canvas and CSS is used to provide various functionality on web page.

#include <windows.h>
#include <stdio.h>
#include <winsock2.h>
#include "ADCLib.h"
#include "gpioLib.h" 

#define IP_ADDRESS                                  "0.0.0.0"                             ///< IP address of module
#define PORT_NUMBER                                 10000                                 ///< PORT number used for web server
#define GPIO                                        -1
#define REC_BUFFER                                  300                                   ///< size of receive buffer
#define RES_BUFFER_SIZE                             24000                                 ///< size of response buffer
#define RES_BUFFER_SIZE_2                           23000
#define COLOUR_BUFFER_SIZE                          20                                    ///< size of colour buffer for button
#define IMAGE_HEADER_SIZE                           1000
#define HTML_HEADER_SIZE                            600
#define TEMPORARY_BUFFER_SIZE                       1000
#define TEMPORARY_BUFFER_SIZE_2                     2000

#define ADC_CHANNEL_NUMBER                          2
#define NUMBER_OF_SAMPLES                           5                                     ///< number of samples of Analog input

#pragma comment(lib, "Ws2.lib")                                                           ///< linking winsock library

int wmain()
{
    
    CHAR faviconHeaderString[] = "HTTP/1.1 204 No Content\r\nContent-Length: 0\r\nConnection: keep-alive\r\n\r\n";                                                             ///< "204 No Content" response header string, used for favicon request
    CHAR headerString[] = "HTTP/1.1 200 OK\r\nConnection: keep-alive\r\nContent-type: text/html\r\nContent-Length:%d\r\n\r\n";                                                 ///< response header string for web page
    CHAR imageHeaderStringTemporary[] = "HTTP/1.1 200 OK\r\nServer: Microsoft-WinCE/7.00\r\nContent-Length:%d\r\nConnection: keep-alive\r\nContent-type: image/gif\r\n\r\n";   ///< response header string for image
    CHAR imageHeaderString[IMAGE_HEADER_SIZE] = {0};                                      ///< string used for final image header
    CHAR finalHeaderString[HTML_HEADER_SIZE] = {0};                                       ///< string used for final HTML header
    CHAR buttonString1[COLOUR_BUFFER_SIZE] = "Opt1=0/Opt1";                               ///< string for output button status
    CHAR buttonString2[COLOUR_BUFFER_SIZE] = "Opt2=0/Opt2";
    CHAR buttonString3[COLOUR_BUFFER_SIZE] = "Opt3=0/Opt3";
    CHAR buttonString4[COLOUR_BUFFER_SIZE] = "Opt4=0/Opt4";
    CHAR recBuffer[55] = {0};                                                             ///< buffer to receive request from client
    CHAR recBufferTemporary[REC_BUFFER] = {0};
    CHAR *htmlBuffer = NULL;                                                              ///< buffer to contain HTML Page
    CHAR *htmlQuery = NULL;
    CHAR *htmlQueryFinal = NULL;                                                          ///< string for Final response
    BYTE *imageBuffer = NULL;                                                             ///< buffer to contain image content
    UINT numberOfSamples = NUMBER_OF_SAMPLES;
    INT retVal = 0;
    BOOL buttonFlag1 = FALSE;                                                             ///< flag for button state, ON or OFF
    BOOL buttonFlag2 = FALSE;
    BOOL buttonFlag3 = FALSE;
    BOOL buttonFlag4 = FALSE;
    BOOL previousInputState1 = FALSE;                                                     ///< flag for previous input status, ON or OFF 
    BOOL previousInputState2 = FALSE;
    BOOL previousInputState3 = FALSE;
    BOOL previousInputState4 = FALSE;
    BOOL currentInputState1 = FALSE;                                                      ///< flag for current input status, ON or OFF
    BOOL currentInputState2 = FALSE;
    BOOL currentInputState3 = FALSE;
    BOOL currentInputState4 = FALSE;
    BOOL outputUpdateFlag1 = FALSE;                                                       ///< flag to signal status of output to be send in update string  
    BOOL outputUpdateFlag2 = FALSE;                                                        
    BOOL outputUpdateFlag3 = FALSE;
    BOOL outputUpdateFlag4 = FALSE;
    DWORD outputUpdateCount1 = 0;                                                         ///< temporary counter for output status that is send in update string
    DWORD outputUpdateCount2 = 0;                                                          
    DWORD outputUpdateCount3 = 0;
    DWORD outputUpdateCount4 = 0;
    DWORD htmlSize = 0;
    DWORD tempCount = 0;
    DWORD rawAdcValue = 0;
    DWORD imageFileSize = 0;
    DWORD temperatureValue = 0;
    DWORD dataBufferSize = 30000;
    DWORD tcpNoDelay = 1;
    DWORD outputPin1 = 86;                                                                ///< LED 1 connected to SODIMM pin 86 
    DWORD outputPin2 = 88;                                                                ///< LED 2 connected to SODIMM pin 88 
    DWORD outputPin3 = 90;                                                                ///< LED 3 connected to SODIMM pin 90 
    DWORD outputPin4 = 92;                                                                ///< LED 4 connected to SODIMM pin 92 
    DWORD inputPin1 = 28;                                                                 ///< switch 1 connected to SODIMM pin 28
    DWORD inputPin2 = 30;                                                                 ///< switch 2 connected to SODIMM pin 30
    DWORD inputPin3 = 67;                                                                 ///< switch 3 connected to SODIMM pin 67
    DWORD inputPin4 = 100;                                                                ///< switch 4 connected to SODIMM pin 100
    DWORD inputPin5 = 45;                                                                 ///< switch 5 connected to SODIMM pin 45

    FILE *htmlFile = NULL;                                                                ///< pointer to HTML file
    FILE *imageFile = NULL;                                                               ///< pointer to image file
    WSADATA windowsSocket;                                                                ///< structure contains information about the Windows Sockets implementation
    struct sockaddr_in tServerInfo;                                                       ///< structure contains information about socket address
    SOCKET listeningSocket;                                                               ///< socket that is bound to a specific transport service provider
    SOCKET client;

    InitGPIOLib();                                                                        ///< initialize GPIO library for module
    /// set SODIMM pin as output
    SetPinAltFn(outputPin1, GPIO, DIR_OUT);
    SetPinAltFn(outputPin2, GPIO, DIR_OUT);
    SetPinAltFn(outputPin3, GPIO, DIR_OUT);
    SetPinAltFn(outputPin4, GPIO, DIR_OUT);
    /// Set SODIMM pin as input
    SetPinAltFn(inputPin1, GPIO, DIR_IN);
    SetPinAltFn(inputPin2, GPIO, DIR_IN);
    SetPinAltFn(inputPin3, GPIO, DIR_IN);
    SetPinAltFn(inputPin4, GPIO, DIR_IN);
    SetPinAltFn(inputPin5, GPIO, DIR_IN);

    if (!Ac97Init()){                                                                     ///< initialize AC97 and get a MUTEX for the AC97 access (Touch panel and Sound driver use AC97 too)
        printf("Could not initialize codec\n");
    }

    retVal = WSAStartup(MAKEWORD(2,2), &windowsSocket);                                   ///< initiating use of ws2.dll(library' used for socket programming) with winsock Version 2.2
    if (retVal != 0)                                                                      ///< if WSAStartup failed to initialize
    {
        printf("WSAStartup failed with error code: %d\n", retVal);
        exit(1);
    }
    listeningSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);                          ///< socket creation
    if (listeningSocket == INVALID_SOCKET)
    {
        printf("\nsocket creation failed with error code : %d", WSAGetLastError());       ///< error message
        WSACleanup();
        exit(1);
    }

    /// socket binding
    tServerInfo.sin_family = AF_INET;                                                     ///< tcp/udp socket
    tServerInfo.sin_addr.s_addr = inet_addr(IP_ADDRESS);                                  ///< IP address assignment to Colibri module
    tServerInfo.sin_port = htons(PORT_NUMBER);                                            ///< port number:10000 used for communication
    retVal = bind(listeningSocket,(SOCKADDR *)&tServerInfo, sizeof(struct sockaddr));
    if (retVal == SOCKET_ERROR)
    {
        printf("\nsocket binding failed with error code : %d", WSAGetLastError());        ///< error message
        WSACleanup();
        exit(1);
    }

    retVal = setsockopt(listeningSocket, IPPROTO_TCP, TCP_NODELAY, (char*)&tcpNoDelay, sizeof(DWORD));            ///< disable Nagle algorithm
    retVal = setsockopt(listeningSocket, SOL_SOCKET, SO_SNDBUF, (char*)&dataBufferSize, sizeof(dataBufferSize));  ///< setting send buffer size
    retVal = setsockopt(listeningSocket, SOL_SOCKET, SO_RCVBUF, (char*)&dataBufferSize, sizeof(dataBufferSize));  ///< setting receive buffer size
    retVal = setsockopt(listeningSocket, SOL_SOCKET, SO_LINGER, (char*)"l_linger", sizeof("l_linger"));           ///< for graceful shut-down

    retVal = listen(listeningSocket, 100);                                                ///< put socket in listening state with 100 maximum connection
    if (retVal == SOCKET_ERROR)
    {
        printf("\nsocket listening failed with error code : %d", WSAGetLastError());      ///< error message
        WSACleanup();
        getchar();
        exit(1);
    }
    printf("socket created.\nPress switch 5 to close Web Server\n");

    while (TRUE)
    {
        client = accept(listeningSocket, NULL, NULL);                                     ///< waiting for request from client
        if (client == INVALID_SOCKET)
        {
            printf("\nsocket listening failed with error code : %d", WSAGetLastError());  ///< error message
            WSACleanup();
            getchar();
            exit(1);
        }
        
        retVal = recv(client, recBufferTemporary, REC_BUFFER, 0);                         ///< receiving request from client
        recBufferTemporary[retVal] = 0;
        strncpy_s(recBuffer, 55, recBufferTemporary, 50);                                 ///< obtain request string, 55 is size of destination buffer

        if (strstr(recBuffer, "GET / HTTP/1.1"))                                          ///< if client request for Web Page
        {
            htmlSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                ///< allocate a block of memory to htmlQueryFinal
            htmlBuffer = (CHAR*)calloc(RES_BUFFER_SIZE_2, sizeof(BYTE));                  ///< allocate a block of memory to htmlBuffer
            htmlQuery = (CHAR*)calloc(RES_BUFFER_SIZE_2, sizeof(BYTE));                   ///< allocate a block of memory to htmlQuery

            fopen_s(&htmlFile, "\\Program Files\\webserverdemo3\\webpage.htm", "rb");     ///< open HTML web page
            fseek(htmlFile, 0 , SEEK_END);
            htmlSize = ftell(htmlFile);                                                   ///< obtain size of web page
            fseek(htmlFile, 0 , SEEK_SET);
            fread(htmlBuffer, 1, htmlSize , htmlFile);
            strcpy_s(htmlQuery, RES_BUFFER_SIZE_2, htmlBuffer);                           ///< copy web page on buffer

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, htmlSize);            ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s%s", finalHeaderString, htmlQuery);  ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                  ///< send final HTML page to client
            /// set output pin High or Low according to respective flag
            SetPinLevel(outputPin1, buttonFlag1);
            SetPinLevel(outputPin2, buttonFlag2);
            SetPinLevel(outputPin3, buttonFlag3);
            SetPinLevel(outputPin4, buttonFlag4);
            /// enable output update flag, so that output status can be send to new client
            outputUpdateFlag1 = TRUE;
            outputUpdateFlag2 = TRUE;
            outputUpdateFlag3 = TRUE;
            outputUpdateFlag4 = TRUE;
			/// set previous input status to default
            previousInputState1 = FALSE;
            previousInputState2 = FALSE;
            previousInputState3 = FALSE;
            previousInputState4 = FALSE;
            Sleep(30);
            free(htmlQueryFinal);                                                         ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            free(htmlBuffer);                                                             ///< deallocate the allocated block of memory to htmlBuffer
            htmlBuffer = NULL;
            free(htmlQuery);                                                              ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            fclose(htmlFile);
        }
        else if (strstr(recBuffer, "favicon"))                                            ///< if client request for "favicon"
        {
            send(client, faviconHeaderString, strlen(faviconHeaderString), 0);            ///< send "204 No content" response code to client
        }
        else if (strstr(recBuffer, "updateStatus"))                                       ///< if client request for "updateStatus"
        {
            rawAdcValue = 0;
            temperatureValue = 0;
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));        ///< allocate a block of memory to htmlQueryFinal
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));               ///< allocate a block of memory to htmlQuery

            ReadADCChannel(ADC_CHANNEL_NUMBER, &rawAdcValue, numberOfSamples);            ///< read input from ADC channel
            temperatureValue = (DWORD)((MicrovoltsPerLsb(ADC_CHANNEL_NUMBER) * rawAdcValue)/(10000.0 * numberOfSamples));
            sprintf_s(htmlQuery, TEMPORARY_BUFFER_SIZE,"ADC=%d/ADC", temperatureValue);   ///< concatenate temperature value into update string
           
            /// if output 1 status (high or low) is changed, then add its current status into next 4 update strings
            if ((outputUpdateFlag1 == TRUE) || ((outputUpdateCount1 < 5) && (outputUpdateCount1 > 0)))
            {
                outputUpdateCount1++;
                if (buttonFlag1== TRUE)
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt1=1/Opt1");
                else
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt1=0/Opt1");
            }
            /// if output 2 status (high or low) is changed, then add its current status into next 4 update strings
            if ((outputUpdateFlag2 == TRUE) || ((outputUpdateCount2 < 5) && (outputUpdateCount2 > 0)))
            {
                outputUpdateCount2++;
                if (buttonFlag2== TRUE)
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt2=1/Opt2");
                else
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt2=0/Opt2");
            }
            /// if output 3 status (high or low) is changed, then add its current status into next 4 update strings
            if ((outputUpdateFlag3 == TRUE) || ((outputUpdateCount3 < 5) && (outputUpdateCount3 > 0)))
            {
                outputUpdateCount3++;
                if (buttonFlag3== TRUE)
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt3=1/Opt3");
                else
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt3=0/Opt3");
            }
            /// if output 4 status (high or low) is changed, then add its current status into next 4 update strings
            if ((outputUpdateFlag4 == TRUE) || ((outputUpdateCount4 < 5) && (outputUpdateCount4 > 0)))
            {
                outputUpdateCount4++;
                if (buttonFlag4== TRUE)
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt4=1/Opt4");
                else
                    strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Opt4=0/Opt4");
            }
            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                   ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);          ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                                  ///< send final HTML page to client

            /// clear output count if they exceed 5
            if (outputUpdateCount1 >= 5)
            {
                outputUpdateCount1 = 0;
                outputUpdateFlag1 = FALSE;
            }
            if (outputUpdateCount2 >= 5)
            {
                outputUpdateCount2 = 0;
                outputUpdateFlag2 = FALSE;
            }
            if (outputUpdateCount3 >= 5)
            {
                outputUpdateCount3 = 0;
                outputUpdateFlag3 = FALSE;
            }
            if (outputUpdateCount4 >= 5)
            {
                outputUpdateCount4 = 0;
                outputUpdateFlag4 = FALSE;
            }
            free(htmlQuery);                                                                       ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }
        else if (strstr(recBuffer, "Tordy.jpg"))                                                   ///< if client request for "Tordy.jpg"
        {
            imageFileSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\tordy.jpg","rb");        ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET); 
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(30);
            free(imageBuffer);                                                                     ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);
        }
        else if (strstr(recBuffer, "Gauge1.jpg"))                                                  ///< if client request for "Gauge1.jpg"
        {
            imageFileSize = 0;
            tempCount = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\gauge1.jpg","rb");       ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(40);
            free(imageBuffer);                                                                     ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);
        }
        else if (strstr(recBuffer, "Gauge2.jpg"))                                                  ///< if client request for "Gauge2.jpg"
        {
            imageFileSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\gauge2.jpg","rb");       ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(40);
            free(imageBuffer);                                                                     ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);

        }
        else if (strstr(recBuffer, "Overlay1.png"))                                                ///< if client request for "Overlay1.png"
        {
            imageFileSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\overlay1.png","rb");     ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(10);
            free(imageBuffer);                                                                     ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);

        }
        else if (strstr(recBuffer, "Overlay2.png"))                                                ///< if client request for "Overlay2.png"
        {
            imageFileSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\overlay2.png","rb");     ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(10);
            free(imageBuffer);                                                                     ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                  ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);
        }
        else if (strstr(recBuffer, "ToradexLogo.gif"))                                             ///< if client request for "ToradexLogo.gif"
        {
            imageFileSize = 0;
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                         ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\toradexlogo.gif","rb");
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                      ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);         ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                 ///< concatenate image data in response string
            
            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);              ///< send final image response string to client
            Sleep(10);
            free(imageBuffer);                                                                      ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                   ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);

        }
        else if (strstr(recBuffer, "Needle1.png"))                                                  ///< if client request for "Needle1.png"
        {
            imageFileSize = 0; 
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                          ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\needle1.png","rb");       ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                       ///< obtain Image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);          ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                 ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                  ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);               ///< send final image response string to client
            Sleep(1);
            free(imageBuffer);                                                                      ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                   ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);
        }
        else if (strstr(recBuffer, "Needle2.png"))                                                  ///< if client request for "Needle2.png"
        {
            imageFileSize = 0; 
            htmlQueryFinal = (CHAR*)calloc(RES_BUFFER_SIZE, sizeof(CHAR));                          ///< allocate a block of memory to htmlQueryFinal
            fopen_s(&imageFile, "\\Program Files\\webserverdemo3\\Images\\needle2.png","rb");       ///< open image file
            fseek(imageFile, 0, SEEK_END);
            imageFileSize = ftell(imageFile);                                                       ///< obtain image size
            fseek(imageFile, 0, SEEK_SET);
            imageBuffer = (BYTE*)calloc(imageFileSize, sizeof(BYTE));
            fread(imageBuffer, sizeof(BYTE), imageFileSize, imageFile);

            sprintf_s(imageHeaderString, IMAGE_HEADER_SIZE, imageHeaderStringTemporary, imageFileSize);          ///< modify image header with size of image
            sprintf_s(htmlQueryFinal, RES_BUFFER_SIZE, "%s", imageHeaderString);                                 ///< add image header in response string
            for (tempCount = strlen(imageHeaderString); tempCount < (strlen(imageHeaderString) + imageFileSize); tempCount++)
                htmlQueryFinal[tempCount] = imageBuffer[tempCount - strlen(imageHeaderString)];                  ///< concatenate image data in response string

            retVal = send(client, htmlQueryFinal, (strlen(imageHeaderString) + imageFileSize), 0);               ///< send final image response string to client
            Sleep(3);
            free(imageBuffer);                                                                      ///< deallocate the allocated block of memory to htmlBuffer
            imageBuffer = NULL;
            free(htmlQueryFinal);                                                                   ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            fclose(imageFile);
            Sleep(1);
        }

        else if (strstr(recBuffer, "Button1.htm"))                                                   ///< if client Request for response, when button 1 is pressed
        {
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));                          ///< allocate a block of memory to htmlQuery
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));                   ///< allocate a block of memory to htmlQueryFinal
            if (buttonFlag1 == TRUE)                                                                 ///< if button was ON, switch it OFF
            {
                buttonFlag1 = FALSE;
                strcpy_s(buttonString1, COLOUR_BUFFER_SIZE, "Opt1=0/Opt1");
            }
            else                                                                                     ///< if button was OFF, switch it ON
            {
                buttonFlag1 = TRUE;
                strcpy_s(buttonString1, COLOUR_BUFFER_SIZE, "Opt1=1/Opt1");
            }
            SetPinLevel(outputPin1, buttonFlag1);                                                    ///< set output 1 as High or Low according to flag status
            outputUpdateFlag1 = TRUE;
            strcpy_s(htmlQuery, TEMPORARY_BUFFER_SIZE, buttonString1);  

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                     ///< modify header string with size of HTML page 
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);            ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);

            free(htmlQuery);                                                                         ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            free(htmlQueryFinal);                                                                    ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }
        else if (strstr(recBuffer, "Button2.htm"))                                                   ///< if client Request for response, when button 2 is pressed
        {
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));                   ///< allocate a block of memory to htmlQueryFinal
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));                          ///< allocate a block of memory to htmlQuery
            if (buttonFlag2 == TRUE)                                                                 ///< if Button was ON, switch it OFF
            {
                buttonFlag2 = FALSE;
                strcpy_s(buttonString2, COLOUR_BUFFER_SIZE, "Opt2=0/Opt2");
            }
            else                                                                                     ///< if button was OFF, switch it ON
            {
                buttonFlag2 = TRUE;
                strcpy_s(buttonString2, COLOUR_BUFFER_SIZE, "Opt2=1/Opt2");
            }
            SetPinLevel(outputPin2, buttonFlag2);                                                    ///< set output 2 as High or Low according to flag status
            outputUpdateFlag2 = TRUE;
            strcpy_s(htmlQuery, TEMPORARY_BUFFER_SIZE, buttonString2);

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                     ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);            ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);
            
            free(htmlQuery);                                                                         ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL; 
            free(htmlQueryFinal);                                                                    ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }
        else if (strstr(recBuffer, "Button3.htm"))                                                   ///< if client request for response, when button 3 is pressed
        {
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));                   ///< allocate a block of memory to htmlQueryFinal
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));                          ///< allocate a block of memory to htmlQuery
            if (buttonFlag3 == TRUE)                                                                 ///< if Button was ON, switch it OFF
            {   
                buttonFlag3 = FALSE;
                strcpy_s(buttonString3, COLOUR_BUFFER_SIZE, "Opt3=0/Opt3");
            }
            else                                                                                     ///< if button was OFF, switch it ON
            {
                buttonFlag3 = TRUE;
                strcpy_s(buttonString3, COLOUR_BUFFER_SIZE, "Opt3=1/Opt3");
            }
            SetPinLevel(outputPin3, buttonFlag3);                                                    ///< set output 3 as High or Low according to flag status
            outputUpdateFlag3 = TRUE;
            strcpy_s(htmlQuery, TEMPORARY_BUFFER_SIZE, buttonString3);  

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                     ///< modify header string with size of HTML Page 
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);            ///< concatenate Header and HTML Page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);

            free(htmlQuery);                                                                         ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            free(htmlQueryFinal);                                                                    ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }
        else if (strstr(recBuffer, "Button4.htm"))                                                   ///< if client Request for response, when Button 4 is pressed
        {
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));                   ///< allocate a block of memory to htmlQueryFinal
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));                          ///< allocate a block of memory to htmlQuery
            if (buttonFlag4 == TRUE)                                                                 ///< if button was ON, switch it OFF
            {
                buttonFlag4 = FALSE;
                strcpy_s(buttonString4, COLOUR_BUFFER_SIZE, "Opt4=0/Opt4");
            }
            else                                                                                    ///< if button was OFF, switch it ON
            {
                buttonFlag4 = TRUE;
                strcpy_s(buttonString4, COLOUR_BUFFER_SIZE, "Opt4=1/Opt4");
            }
            SetPinLevel(outputPin4, buttonFlag4);                                                   ///< set output 4 as High or Low according to flag status
            outputUpdateFlag4 = TRUE;
            strcpy_s(htmlQuery, TEMPORARY_BUFFER_SIZE, buttonString4);

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                     ///< modify header string with size of HTML Page 
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);            ///< concatenate Header and HTML Page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);                       ///< send final HTML Page to client

            free(htmlQuery);                                                                        ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            free(htmlQueryFinal);                                                                   ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }
        else if (strstr(recBuffer, "Ipt.htm"))                                                      ///< if client request for input status 
        {
            htmlQuery = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE, sizeof(BYTE));                         ///< allocate a block of memory to htmlQuery
            htmlQueryFinal = (CHAR*)calloc(TEMPORARY_BUFFER_SIZE_2, sizeof(CHAR));                  ///< allocate a block of memory to htmlQueryFinal
            /// read input pin level, HIGH or LOW
            currentInputState1 = GetPinLevel(inputPin1);
            currentInputState2 = GetPinLevel(inputPin2);
            currentInputState3 = GetPinLevel(inputPin3);
            currentInputState4 = GetPinLevel(inputPin4);
            /// concatenate input status into response string
            if (currentInputState1 == TRUE)
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt1=1/Ipt1");
            else
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt1=0/Ipt1");

            if (currentInputState2 == TRUE)
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt2=1/Ipt2");
            else
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt2=0/Ipt2");

            if (currentInputState3 == TRUE)
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt3=1/Ipt3");
            else
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt3=0/Ipt3");

            if (currentInputState4 == TRUE)
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt4=1/Ipt4");
            else
                strcat_s(htmlQuery, TEMPORARY_BUFFER_SIZE, "Ipt4=0/Ipt4");

            sprintf_s(finalHeaderString, HTML_HEADER_SIZE, headerString, strlen(htmlQuery));                     ///< modify header string with size of HTML page
            sprintf_s(htmlQueryFinal, TEMPORARY_BUFFER_SIZE_2, "%s%s", finalHeaderString, htmlQuery);            ///< concatenate header and HTML page
            retVal = send(client, htmlQueryFinal, strlen(htmlQueryFinal), 0);

            free(htmlQuery);                                                                        ///< deallocate the allocated block of memory to htmlQuery
            htmlQuery = NULL;
            free(htmlQueryFinal);                                                                   ///< deallocate the allocated block of memory to htmlQueryFinal
            htmlQueryFinal = NULL;
            Sleep(1);
        }

        if (retVal == SOCKET_ERROR)
        {
            printf("Error, closing connection");
            MessageBox(0, L"could not send message to client", L"error", 0);
            WSACleanup();
            exit(1);
        }
        shutdown(client, SD_BOTH);                                                                  ///< closing both sending and receiving of client
        closesocket(client);
        if (GetPinLevel(inputPin5) == TRUE)                                                         ///< if switch 5 is pressed and close web server
        {
            printf("Web server closed");
            WSACleanup();
            exit(0);
        }
    }

    closesocket(listeningSocket);                                                                   ///< closing listening_socket 
    WSACleanup();                                                                                   ///< releasing use of ws2.dll(library' used for socket programming) with winsock Version 2.2
    DeInitGPIOLib();                                                                                ///< releasing GPIO library
    return 0;
}
